package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.Statement;
import org.eclipse.jdt.internal.compiler.codegen.BranchLabel;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.flow.LabelFlowContext;
import org.eclipse.jdt.internal.compiler.flow.UnconditionalFlowInfo;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.Scope;

public class LabeledStatement extends Statement {
  public Statement statement;
  
  public char[] label;
  
  public BranchLabel targetLabel;
  
  public int labelEnd;
  
  int mergedInitStateIndex = -1;
  
  public LabeledStatement(char[] label, Statement statement, long labelPosition, int sourceEnd) {
    this.statement = statement;
    if (statement instanceof org.eclipse.jdt.internal.compiler.ast.EmptyStatement)
      statement.bits |= 0x1; 
    this.label = label;
    this.sourceStart = (int)(labelPosition >>> 32L);
    this.labelEnd = (int)labelPosition;
    this.sourceEnd = sourceEnd;
  }
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    if (this.statement == null)
      return flowInfo; 
    LabelFlowContext labelContext;
    FlowInfo statementInfo = this.statement.analyseCode(
        currentScope, 
        (FlowContext)(labelContext = 
        new LabelFlowContext(
          flowContext, 
          (ASTNode)this, 
          this.label, 
          this.targetLabel = new BranchLabel(), 
          currentScope)), 
        flowInfo);
    boolean reinjectNullInfo = ((statementInfo.tagBits & 0x3) != 0 && (
      labelContext.initsOnBreak.tagBits & 0x3) == 0);
    UnconditionalFlowInfo unconditionalFlowInfo = statementInfo.mergedWith(labelContext.initsOnBreak);
    if (reinjectNullInfo)
      unconditionalFlowInfo.addInitializationsFrom((FlowInfo)flowInfo.unconditionalFieldLessCopy())
        .addInitializationsFrom((FlowInfo)labelContext.initsOnBreak.unconditionalFieldLessCopy()); 
    this.mergedInitStateIndex = 
      currentScope.methodScope().recordInitializationStates((FlowInfo)unconditionalFlowInfo);
    if ((this.bits & 0x40) == 0)
      currentScope.problemReporter().unusedLabel(this); 
    return (FlowInfo)unconditionalFlowInfo;
  }
  
  public ASTNode concreteStatement() {
    return (ASTNode)this.statement;
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream) {
    if ((this.bits & Integer.MIN_VALUE) == 0)
      return; 
    int pc = codeStream.position;
    if (this.targetLabel != null) {
      this.targetLabel.initialize(codeStream);
      if (this.statement != null)
        this.statement.generateCode(currentScope, codeStream); 
      this.targetLabel.place();
    } 
    if (this.mergedInitStateIndex != -1) {
      codeStream.removeNotDefinitelyAssignedVariables((Scope)currentScope, this.mergedInitStateIndex);
      codeStream.addDefinitelyAssignedVariables((Scope)currentScope, this.mergedInitStateIndex);
    } 
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  
  public StringBuffer printStatement(int tab, StringBuffer output) {
    printIndent(tab, output).append(this.label).append(": ");
    if (this.statement == null) {
      output.append(';');
    } else {
      this.statement.printStatement(0, output);
    } 
    return output;
  }
  
  public void resolve(BlockScope scope) {
    if (this.statement != null)
      this.statement.resolve(scope); 
  }
  
  public void traverse(ASTVisitor visitor, BlockScope blockScope) {
    if (visitor.visit(this, blockScope) && 
      this.statement != null)
      this.statement.traverse(visitor, blockScope); 
    visitor.endVisit(this, blockScope);
  }
  
  public boolean doesNotCompleteNormally() {
    if (this.statement.breaksOut(this.label))
      return false; 
    return this.statement.doesNotCompleteNormally();
  }
  
  public boolean completesByContinue() {
    return this.statement instanceof org.eclipse.jdt.internal.compiler.ast.ContinueStatement;
  }
}
